@@ -87,9 +87,20 @@ class TrafficItem(CustomField):
     }
 
     _IPV4 = {
-        'src' : 'ipv4.header.srcIp',
-        'dst' : 'ipv4.header.dstIp',
+        'version' : 'ipv4.header.version',
+        'header_length' : 'ipv4.header.headerLength',
         'priority' : '_ipv4_priority',
+        'total_length': 'ipv4.header.totalLength',
+        'identification' : 'ipv4.header.identification',
+        'reserved' : 'ipv4.header.flags.reserved',
+        'dont_fragment' : 'ipv4.header.flags.fragment',
+        'more_fragments' : 'ipv4.header.flags.lastFragment',
+        'fragment_offset' : 'ipv4.header.fragmentOffset',
+        'time_to_live' : 'ipv4.header.ttl',
+        'protocol' : 'ipv4.header.protocol',
+        'header_checksum' : 'ipv4.header.checksum',
+        'src': 'ipv4.header.srcIp',
+        'dst': 'ipv4.header.dstIp',
     }
 
     _TOS = {
@@ -391,11 +402,14 @@ class TrafficItem(CustomField):
             regex = '^(%s)$' % '|'.join(request.flow_names)
         if request.state == 'start':
             self._api._traffic_item.find(Name=regex, State='unapplied')
-            self._api._ixnetwork.StartAllProtocols('sync')
+            if len(self._api._topology.find()) > 0:
+                self._api._ixnetwork.StartAllProtocols('sync')
             if len(self._api._traffic_item) > 0:
                 self._api._traffic_item.Generate()
                 self._api._traffic.Apply()
             self._api._start_capture()
+            self._api._ixnetwork.ClearStats(
+                ['waitForPortStatsRefresh', 'waitForTrafficStatsRefresh'])
         self._api._traffic_item.find(Name=regex)
         if request.state == 'start':
             self._api._traffic_item.StartStatelessTrafficBlocking()
