@@ -1,5 +1,6 @@
 import json
 from jsonpath_ng.ext import parse
+import time
 
 
 class Vport(object):
@@ -113,63 +114,114 @@ class Vport(object):
     def _create_capture(self):
         """Overwrite any capture settings
         """
-        vports = self._api.select_vports()
+        if self._api.config.captures is None:
+            self._api.config.captures = []
         imports = []
-        for port in self._api.config.ports:
+        vports = self._api.select_vports()
+        for vport in vports.values():
             capture = {
-                'xpath': vports[port.name]['xpath'] + '/capture',
+                'xpath': vport['xpath'] + '/capture',
                 'captureMode': 'captureTriggerMode',
                 'hardwareEnabled': False,
                 'softwareEnabled': False
             }
-            pallette = {
-                'xpath': capture['xpath'] + '/filterPallette'
-            }
-            filter = {
-                'xpath': capture['xpath'] + '/filter'
-            }
-            trigger = {
-                'xpath': capture['xpath'] + '/trigger'
-            }
-            if port.capture is not None and port.capture.basic is not None:
-                capture['hardwareEnabled'] = True
-                filter['captureFilterEnable'] = True
-                trigger['captureTriggerEnable'] = True
-                filter['captureFilterEnable'] = True
-                for basic in port.capture.basic:
-                    if basic.choice == 'mac_address' and basic.mac_address.mac == 'source':
-                        pallette['SA1'] = basic.mac_address.filter
-                        pallette['SAMask1'] = basic.mac_address.mask
-                        filter['captureFilterSA'] = 'notAddr1' if basic.not_operator is True else 'addr1'
-                        trigger['triggerFilterSA'] = filter['captureFilterSA']
-                    elif basic.choice == 'mac_address' and basic.mac_address.mac == 'destination':
-                        pallette['DA1'] = basic.mac_address.filter
-                        pallette['DAMask1'] = basic.mac_address.mask
-                        filter['captureFilterDA'] = 'notAddr1' if basic.not_operator is True else 'addr1'
-                        trigger['triggerFilterDA'] = filter['captureFilterDA']
-                    elif basic.choice == 'custom':
-                        pallette['pattern1'] = basic.custom.filter
-                        pallette['patternMask1'] = basic.custom.mask
-                        pallette['patternOffset1'] = basic.custom.offset
-                        filter['captureFilterPattern'] = 'notPattern1' if basic.not_operator is True else 'pattern1'
-                        trigger['triggerFilterPattern'] = filter['captureFilterPattern']
             imports.append(capture)
-            imports.append(pallette)
-            imports.append(filter)
-            imports.append(trigger)
+        for capture_item in self._api.config.captures:
+            for port_name in capture_item.port_names:
+                capture = {
+                    'xpath': vports[port_name]['xpath'] + '/capture',
+                    'captureMode': 'captureTriggerMode',
+                    'hardwareEnabled': False,
+                    'softwareEnabled': False
+                }
+                pallette = {
+                    'xpath': capture['xpath'] + '/filterPallette'
+                }
+                filter = {
+                    'xpath': capture['xpath'] + '/filter'
+                }
+                trigger = {
+                    'xpath': capture['xpath'] + '/trigger'
+                }
+                if capture_item.basic is not None:
+                    capture['hardwareEnabled'] = True
+                    filter['captureFilterEnable'] = True
+                    trigger['captureTriggerEnable'] = True
+                    filter['captureFilterEnable'] = True
+                    for basic in capture_item.basic:
+                        if basic.choice == 'mac_address' and basic.mac_address.mac == 'source':
+                            pallette['SA1'] = basic.mac_address.filter
+                            pallette['SAMask1'] = basic.mac_address.mask
+                            filter['captureFilterSA'] = 'notAddr1' if basic.not_operator is True else 'addr1'
+                            trigger['triggerFilterSA'] = filter['captureFilterSA']
+                        elif basic.choice == 'mac_address' and basic.mac_address.mac == 'destination':
+                            pallette['DA1'] = basic.mac_address.filter
+                            pallette['DAMask1'] = basic.mac_address.mask
+                            filter['captureFilterDA'] = 'notAddr1' if basic.not_operator is True else 'addr1'
+                            trigger['triggerFilterDA'] = filter['captureFilterDA']
+                        elif basic.choice == 'custom':
+                            pallette['pattern1'] = basic.custom.filter
+                            pallette['patternMask1'] = basic.custom.mask
+                            pallette['patternOffset1'] = basic.custom.offset
+                            filter['captureFilterPattern'] = 'notPattern1' if basic.not_operator is True else 'pattern1'
+                            trigger['triggerFilterPattern'] = filter['captureFilterPattern']
+                imports.append(capture)
+                imports.append(pallette)
+                imports.append(filter)
+                imports.append(trigger)
         self._import(imports)
 
+    def _add_hosts(self, HostReadyTimeout):
+        chassis = self._api._ixnetwork.AvailableHardware.Chassis
+        ip_addresses = []
+        for port in self._api.config.ports:
+            if port.location is not None and ';' in port.location:
+                chassis_address = port.location.split(';')[0]
+                chassis.find(Hostname='^(%s)$' % chassis_address)
+                if len(chassis) == 0:
+                    ip_addresses.append(chassis_address)
+        ip_addresses = set(ip_addresses)
+        if len(ip_addresses) == 0:
+            return
+        self._api._ixnetwork.info('Adding location hosts [%s]...' % ', '.join(ip_addresses))
+        for ip_address in ip_addresses:
+            chassis.add(Hostname=ip_address)
+        start_time = time.time()
+        while True:
+            chassis.find(Hostname='^(%s)$' % '|'.join(ip_addresses), State='^ready$')
+            if len(chassis) == len(ip_addresses):
+                break
+            if time.time() - start_time > HostReadyTimeout:
+                raise RuntimeError('After %s seconds, not all location hosts [%s] are reachable' % (HostReadyTimeout, ', '.join(ip_addresses)))
+            time.sleep(2)
+
     def _set_location(self):
+        locations = []
+        self._add_hosts(10)
         vports = self._api.select_vports()
         imports = []
         for port in self._api.config.ports:
             self._api.ixn_objects[port.name] = vports[port.name]['href']
+            location = getattr(port, 'location', None)
             vport = {
                 'xpath': vports[port.name]['xpath'],
-                'location': getattr(port, 'location', None),
+                'location': location
             }
             imports.append(vport)
+            if location is not None and len(location) > 0:
+                locations.append(port.name)
+        self._api._ixnetwork.info('Connecting locations [%s]...' % ', '.join(locations))
         self._import(imports)
+        self._api._ixnetwork.info('Checking location state [%s]...' % ', '.join(locations))
+        start = time.time()
+        timeout = 30
+        while True:
+            self._api._vport.find(Name='^(%s)$' % '|'.join(locations), ConnectionState='^connectedLink')
+            if len(self._api._vport) == len(locations):
+                break
+            if time.time() - start > timeout:
+                raise RuntimeError('After %s seconds, not all locations [%s] are reachable' % (timeout, ', '.join(locations)))
+            time.sleep(2)
 
     def _set_layer1(self):
         """Set the /vport/l1Config/... properties
@@ -294,13 +346,13 @@ class Vport(object):
                 'pfcPauseDelay': pfc.pfc_delay,
                 'pfcPriorityGroups': [
                     -1 if pfc.pfc_class_0 is None else pfc.pfc_class_0,
-                    -1 if pfc.pfc_class_1 is None else pfc.pfc_class_0,
-                    -1 if pfc.pfc_class_2 is None else pfc.pfc_class_0,
-                    -1 if pfc.pfc_class_3 is None else pfc.pfc_class_0,
-                    -1 if pfc.pfc_class_4 is None else pfc.pfc_class_0,
-                    -1 if pfc.pfc_class_5 is None else pfc.pfc_class_0,
-                    -1 if pfc.pfc_class_6 is None else pfc.pfc_class_0,
-                    -1 if pfc.pfc_class_7 is None else pfc.pfc_class_0,
+                    -1 if pfc.pfc_class_1 is None else pfc.pfc_class_1,
+                    -1 if pfc.pfc_class_2 is None else pfc.pfc_class_2,
+                    -1 if pfc.pfc_class_3 is None else pfc.pfc_class_3,
+                    -1 if pfc.pfc_class_4 is None else pfc.pfc_class_4,
+                    -1 if pfc.pfc_class_5 is None else pfc.pfc_class_5,
+                    -1 if pfc.pfc_class_6 is None else pfc.pfc_class_6,
+                    -1 if pfc.pfc_class_7 is None else pfc.pfc_class_7,
                 ],
                 'priorityGroupSize': 'priorityGroupSize-8',
                 'supportDataCenterMode': True
@@ -310,7 +362,7 @@ class Vport(object):
     def _connect(self):
         self._ixn_vport.find(ConnectionState='^((?!connectedLink).)*$')
         try:
-            force_ownership = self._api.config.options.port_options.location_premption
+            force_ownership = self._api.config.options.port_options.location_preemption
         except:
             force_ownership = False
         try:
